@@ -10,3 +10,7 @@ from .model_information import ModelInformation
 from .data_classes.category import CategoryType
 from .globals import GLOBALS
 from .loop import loop
+
+from . import log_conf
+
+log_conf.init()
